
getUsagePerMachine <- function(tasks, taskToMachines, machines, taskUsage) {
    
    usagePerMachine <- merge(tasks, taskToMachines, by=c("Task")) 
    usagePerMachine <- usagePerMachine[,c(1:4,7)]
    
    # Merge the Task Usage data
    usagePerMachine <- merge(usagePerMachine, taskUsage[,c(2:4)], by=c("Task")) 
    
    # Aggregate by Machine
    usagePerMachine <- aggregate(. ~ Machine, usagePerMachine[,c(2:3,5:7)], sum)
    # Add the machine data
    usagePerMachine <- merge(usagePerMachine, machines[,c(1:4)],  by=c("Machine")) 
    
    names(usagePerMachine)[names(usagePerMachine) == "RAM (GB).x"] <- "Tasks RAM (GB)"
    names(usagePerMachine)[names(usagePerMachine) == "CPU (GHz).x"] <- "Tasks CPU (GHz)"
    names(usagePerMachine)[names(usagePerMachine) == "RAM (GB) usage"] <- "Tasks usage RAM (GB)"
    names(usagePerMachine)[names(usagePerMachine) == "MIPS usage"] <- "Tasks usage MIPS"
    names(usagePerMachine)[names(usagePerMachine) == "Number of Cores"] <- "Machine Number of Cores"
    names(usagePerMachine)[names(usagePerMachine) == "CPU (GHz).y"] <- "Machine CPU (GHz)"
    names(usagePerMachine)[names(usagePerMachine) == "RAM (GB).y"] <- "Machine RAM (GB)"
    
    usagePerMachine <- usagePerMachine[,c(1,6:8,2:5)]
    
    usagePerMachine$`Tasks Requested CPU Util` <- usagePerMachine$`Tasks CPU (GHz)` / 
        (usagePerMachine$`Machine Number of Cores` * usagePerMachine$`Machine CPU (GHz)`)
    
    usagePerMachine$`Tasks Usage CPU Util` <- (usagePerMachine$`Tasks usage MIPS` / 1000) / 
        (usagePerMachine$`Machine Number of Cores` * usagePerMachine$`Machine CPU (GHz)`)
    
    usagePerMachine$`Tasks Requested RAM Util` <- usagePerMachine$`Tasks RAM (GB)` / 
        (usagePerMachine$`Machine RAM (GB)`)
    
    usagePerMachine$`Tasks Usage RAM Util` <- usagePerMachine$`Tasks usage RAM (GB)` / 
        (usagePerMachine$`Machine RAM (GB)`)
    
    return(usagePerMachine)
}

writeUsageTasksFile <- function(taskUsage, name = "TaskUsage.csv") {
    # I need to multiply by 1k to convert form CPU to MIPS which is
    # what the model expects.
    write.csv(taskUsage[, c("time", "Task", "MIPS usage")], 
              file = paste0(resourcesDir, "/", name), 
              row.names=FALSE)
}

getTaskUsageForTime <- function(taskUsage, startTime, endTime) {
    subsetTaskUsage <- taskUsage[taskUsage$time >= startTime & 
                                     taskUsage$time <= endTime,]
    subsetTaskUsage <- merge(taskToMachines, subsetTaskUsage, by = c("Task"))
    return (subsetTaskUsage[order(subsetTaskUsage$Task, subsetTaskUsage$time),])
}

getInitialTaskUsage <- function(taskUsage) {
    initialTaskUsage <- taskUsage[taskUsage$time == 0,]
    return (initialTaskUsage)
}

generateTaskUsageData <- function(data) {
    data$time <- as.integer((data$`start time` + 
                ((data$`end time` - data$`start time`)/2))/1000000 + 0.5) - 600
    data$time[data$`start time` == 600000000] <- 0
    data$`task ID` <- paste0((data$`job ID`), "-", data$`task index`)
    taskUsage <- data[,c("time", "task ID", "CPU rate", "canonical memory usage",
                "maximum CPU rate", "maximum memory usage")]
    taskUsage <- na.omit(taskUsage)
    taskUsage <- taskUsage[!duplicated(taskUsage[1:2]),]
    
    # map to 4 cores at 2660 GHz
    taskUsage$`MIPS usage` <- as.integer((taskUsage$`CPU rate` * 4 * 2660) + 0.5);
    # map to 4GB RAM
    taskUsage$`RAM (GB) usage` <- taskUsage$`canonical memory usage` * 4
    
    taskUsage <- merge(taskUsage, tasks, by=c("task ID"))
    taskUsage <- taskUsage[order(taskUsage$time), c("time", "Task", 
        "MIPS usage", "RAM (GB) usage", 
        "CPU rate", "canonical memory usage", 
        "maximum CPU rate", "maximum memory usage")]
    return (taskUsage)
}

getMachineTaskCounts <- function(taskToMachines) {
    machineTaskCounts <- taskToMachines %>% count(Machine)
    names(machineTaskCounts)[names(machineTaskCounts) == "n"] <- "Number of Tasks"
    return(machineTaskCounts)
}

assignTasksToMachines <- function(machines, tasks) {
    taskToMachines <- tasks[, c("Task", "machine ID")]
    taskToMachines <- taskToMachines[!duplicated(taskToMachines[1:2]),]
    taskToMachines <- merge(taskToMachines, machines, by=c("machine ID"))
    return (taskToMachines[order(taskToMachines$Task),c("Task", "Machine")])
}

writeTasksFile <- function(tasks) {
    # "VM",   "VM Spec"
    write.csv(tasks[order(tasks$Task), c("Task", "Task Type")], 
              file = paste0(resourcesDir, "/Tasks.csv"), row.names=FALSE)
}

writeTaskTypesFile <- function(taskTypes) {
    taskTypes$`MIPS` <- as.integer((taskTypes$`CPU (GHz)` * 1000) + 0.5)
    # "Name", 	"CPU (GHz)", "RAM (GB)"
    write.csv(taskTypes[, c("Task Type", "MIPS", "RAM (GB)")], 
            file = paste0(resourcesDir, "/TaskTypes.csv"), row.names=FALSE)
}

addTaskTypesToTasks <- function(tasks, taskTypes) {
    tasks <- merge(tasks, taskTypes, by=c("CPU (GHz)","RAM (GB)"))
    return(tasks[order(tasks$Task),c(3,1:2,6,4,5)])
}

generateTaskTypes <- function(tasks) {
    taskTypes <- tasks[!duplicated(tasks[2:3]), c("CPU (GHz)", "RAM (GB)")]
    taskTypes <- taskTypes[
        order(taskTypes$`CPU (GHz)`, taskTypes$`RAM (GB)`),]
    taskTypes$`Task Type` <- seq.int(0, nrow(taskTypes) -1)
    return(taskTypes[,c("Task Type", "CPU (GHz)", "RAM (GB)")])
}

# SUBMIT (0): A task or job became eligible for scheduling.
# SCHEDULE (1): A job or task was scheduled on a machine. (It may not start running immediately due to code-shipping time, etc.) For jobs, this occurs the first time any task of the job is scheduled on a machine.
# EVICT(2): A task or job was descheduled because of a higher priority task or job, because the scheduler overcommitted and the actual demand exceeded the machine capacity, because the machine on which it was running became unusable (e.g. taken offline for repairs), or because a disk holding the task’s data was lost.
# FAIL(3): A task or job was descheduled (or, in rare cases, ceased to be eligible for scheduling while it was pending) due to a task failure.
# FINISH(4): A task or job completed normally.
# KILL(5): A task or job was cancelled by the user or a driver program or because another job or task on which this job was dependent died.
# LOST(6): A task or job was presumably terminated, but a record indicating its termination was missing from our source data.
# UPDATE_PENDING(7): A task or job’s scheduling class, resource requirements, or constraints were updated while it was waiting to be scheduled.
# UPDATE_RUNNING(8): A task or job’s scheduling class, resource requirements, or constraints were updated while it was scheduled.

generateTaskData <- function(data, starttime = 0) {
    # Limit to SCHEDULE events only
    tasks <- data[data$`event type`== 1,]
    # Limit to tasks which are already running
    tasks <- tasks[tasks$time <= starttime,]
    # Make a unique task id
    tasks$`task ID` <- paste0((tasks$`job ID`), "-", tasks$`task index`)
    
    # 1 map to 4 cores at 2.66 
    tasks$`CPU (GHz)` <- (tasks$`CPU request` * 4 * 2.66) + 0.5;
    # map to 4GB RAM
    tasks$`RAM (GB)` <- tasks$`memory request` * 4
    
    # Select only interesting fields
    tasks <- tasks[,c("CPU (GHz)", "RAM (GB)", "task ID", "machine ID")]
    
    # Remove and bad data
    tasks <- na.omit(tasks)
    # Remove any duplicates
    tasks <- tasks[!duplicated(tasks[1:4]),]
    
    tasks <- tasks[order(tasks$`task ID`),]
    tasks$`Task` <- seq.int(0, nrow(tasks) - 1)
    return (tasks[,c("Task", "CPU (GHz)", "RAM (GB)", "task ID", "machine ID")])
}

writeMachinesFile <- function(machines) {
    # "Host", "Host Spec"
    write.csv(machines[order(machines$`Machine`), c("Machine", "Machine Type")], 
              file = paste0(resourcesDir, "/Machines.csv"), row.names=FALSE)
}

writeMachinePowerConsumptionFile <- function(machinesTypes) {
    # "Name", "Number of Cores", "CPU (GHz)", "RAM (GB)"
    # "Name",   "0%", "100%"
    # "Host",   175, 250 
    #  100% 135
    machinesPowerConsumption <- machinesTypes[, c("Machine Type", "Number of Cores")]
    machinesPowerConsumption$`0%` <- 
        (machinesPowerConsumption$`Number of Cores` / 4) * 135 * 0.7
    machinesPowerConsumption$`100%` <- 
        (machinesPowerConsumption$`Number of Cores` / 4 )* 135
    write.csv(machinesPowerConsumption[,c("Machine Type", "0%", "100%")], 
              file = paste0(resourcesDir, "/MachinePowerConsumption.csv"), 
              row.names=FALSE)
}

writeMachinesTypesFile <- function(machinesTypes) {
    # "Name", "Number of Cores", "CPU (GHz)", "RAM (GB)"
    write.csv(machinesTypes,
              file = paste0(resourcesDir, "/MachineTypes.csv"), 
              row.names=FALSE)
}

addMachineTypesToMachines <- function(machines, machinesTypes) {
    # df <- subset(df, select = -c(a, c))
    machines <- merge(machines, machinesTypes[,c(1:2,4)],
                      by=c("Number of Cores","RAM (GB)")) 
    return(machines[order(machines$Machine),c(6,1,5,2,7,3)])
}

generateMachineTypes <- function(machines) {
    machineTypes <- machines[!duplicated(machines[2:3]),]
    machineTypes <- machineTypes[
        order(machineTypes$`Number of Cores`, machineTypes$`RAM (GB)`),]
    machineTypes$`Machine Type` <- seq.int(0, nrow(machineTypes) - 1)
    # "Machine Type", "Number of Cores", "CPU (GHz)", "RAM (GB)"
    return(machineTypes[order(machineTypes$`Machine Type`), c(7, 3:5)])
}

# Event Type 0 = ADD: a machine became available to the cluster
# Event Type 1 = REMOVE: a machine was removed from the cluster.
# Event Type 2 = UPDATE: a machine available to the cluster had its 
#                        available resources changed.
generateMachineData <- function(data) {
    # Limit to ADD events only
    machines <- data[data$`event type`== 0, c("machine ID", "CPUs", "Memory")]
    machines <- na.omit(machines)
    machines <- machines[!duplicated(machines[1:3]),]
    machines <- machines[order(machines$`machine ID`),]
    # map to 4 cores at 2660 GHz
    machines$`Number of Cores` <- machines$CPUs * 4
    machines$`CPU (GHz)` <- 2.660
    # map to 4GB RAM
    machines$`RAM (GB)` <- machines$Memory * 4
    machines$`Machine` <- seq.int(0, nrow(machines) - 1)
    # return (machines[, c("Machine", "CPU (GHz)", "RAM (GB)", "machine ID")])
    return (machines[,c(1, 3:7)])
}

readDataFiles <- function(dataFiles, headers, nrows=2000) {
    data <- data.frame()
    for (file in dataFiles) {
        data <- rbind(data, read.csv(file, strip.white = TRUE, 
            comment.char = "#", header = FALSE, nrows = nrows))
    }
    names(data) <- headers
    return(data)
}

listCSVFiles <- function(dataDir) {
    path <- file.path(dataDir, "*.csv")
    dataFiles <- Sys.glob(path)
    return(dataFiles)
}
